Package: irpfR
Title: Access to the Great Numbers of Brazilian Individual Income Tax
        (IRPF)
Version: 0.1.1
Authors@R: 
    person("Luã", "Braga de Oliveira", , "luabragadeoliveira@gmail.com", role = c("aut", "cre"))
Description: Provides functions to download, clean, and structure open data from the Brazilian Federal Revenue (Receita Federal do Brasil - RFB) regarding Personal Income Tax (IRPF) statements. Includes a data dictionary and categorized metadata for several sections such as assets, rights, debts, and income brackets. More information about the data source can be found at <https://dados.gov.br/dados/conjuntos-dados/grandes-nmeros-do-imposto-de-renda-da-pessoa-fsica>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, httr2, janitor, magrittr, readr, tidyr, rlang,
        tidyselect, tibble
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.5)
Language: en-US
URL: https://github.com/luabragadeoliveira/irpfR
BugReports: https://github.com/luabragadeoliveira/irpfR/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-25 18:57:51 UTC; luabr
Author: Luã Braga de Oliveira [aut, cre]
Maintainer: Luã Braga de Oliveira <luabragadeoliveira@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 19:20:08 UTC
Built: R 4.5.2; ; 2026-04-28 20:13:08 UTC; unix

Package: irpfR
Title: Access to the Great Numbers of Brazilian Individual Income Tax
        (IRPF)
Version: 0.1.1
Authors@R: 
    person("Luã", "Braga de Oliveira", , "luabragadeoliveira@gmail.com", role = c("aut", "cre"))
Description: Provides functions to download, clean, and structure open data from the Brazilian Federal Revenue (Receita Federal do Brasil - RFB) regarding Personal Income Tax (IRPF) statements. Includes a data dictionary and categorized metadata for several sections such as assets, rights, debts, and income brackets. More information about the data source can be found at <https://dados.gov.br/dados/conjuntos-dados/grandes-nmeros-do-imposto-de-renda-da-pessoa-fsica>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, httr2, janitor, magrittr, readr, tidyr, rlang,
        tidyselect, tibble
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.5)
Language: en-US
URL: https://github.com/luabragadeoliveira/irpfR
BugReports: https://github.com/luabragadeoliveira/irpfR/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-25 18:57:51 UTC; luabr
Author: Luã Braga de Oliveira [aut, cre]
Maintainer: Luã Braga de Oliveira <luabragadeoliveira@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 19:20:08 UTC
Built: R 4.6.0; ; 2026-04-28 20:33:15 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(get_irpf)
export(get_metadata)
export(get_sections)
import(tibble)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
importFrom(tidyselect,where)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# irpfR <img src='man/figures/logo.png' align="right" height="139" />

<!-- badges: start -->

![alt
text](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)
![alt text](https://www.r-pkg.org/badges/version/irpfR)
[![R-CMD-check](https://github.com/luabragadeoliveira/irpfR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/luabragadeoliveira/irpfR/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The goal of `irpfR` is to provide a seamless interface for accessing,
cleaning, and understanding the Personal Income Tax (IRPF) Open Data
provided by the Brazilian Federal Revenue (Receita Federal do Brasil -
RFB). Directly downloading data from the government portal often
involves dealing with inconsistently formatted CSV files, complex
encoding issues, and tricky numerical scales. `irpfR` automates this
engineering work, returning “tidy” data frames ready for economic and
social analysis.

## Features

- **Data Discovery**: Quickly list all available data sections (assets,
  debts, income brackets, etc.).
- **Built-in Dictionary**: Access detailed metadata and attribute
  descriptions without leaving R.
- **Automated Cleaning**: Standardized column names, UTF-8 encoding
  correction, and “tidy” (long) format conversion.
- **Smart Scaling**: Automatic conversion of financial values (from
  millions to absolute BRL) while preserving counts (number of
  taxpayers).

## Installation

You can install the development version of `irpfR` from GitHub with:

``` r
# install.packages("devtools")
devtools::install_github("luabragadeoliveira/irpfR")
```

## Quick Start

### 1. List Available Sections

Not sure what data is available? Use `get_sections()` to see the
catalog:

``` r
library(irpfR)

get_sections()
#> Fetching available sections for IRPF data...
#> # A tibble: 23 × 2
#>    secao                                 descricao                              
#>    <chr>                                 <chr>                                  
#>  1 bens_e_direitos                       Números das Declarações do Imposto de …
#>  2 faixa_de_base_de_calculo_anual        Resumo das declarações por faixa de ba…
#>  3 rendimentos_isentos_e_nao_tributaveis Agregado dos rendimentos isentos e não…
#>  4 residencia_capital                    Resumo das declarações dos contribuint…
#>  5 dividas_e_onus                        Total de valores declarados por tipo d…
#>  6 residencia_uf                         Resumo das declarações dos contribuint…
#>  7 base_calculo_sm_genero                Resumo das declarações por faixa de ba…
#>  8 doacoes_herancas                      Resumo das declarações por faixa de do…
#>  9 rendimento_tributavel_bruto           Resumo das declarações por faixa de re…
#> 10 rendimentos_totais                    Resumo das declarações por faixa de re…
#> # ℹ 13 more rows
```

### 2. Check Metadata

Before downloading, understand what each column in a specific section
means:

``` r
# Example for 'Assets and Rights' (Bens e Direitos)
get_metadata("bens_e_direitos")
#> Fetching metadata for section: 'bens_e_direitos'...
#> # A tibble: 61 × 2
#>    atributo                                              descricao              
#>    <chr>                                                 <chr>                  
#>  1 ano_calendario                                        Informa o ano calendár…
#>  2 acoes_inclusive_as_provenientes_de_linha_telefonica   Ações declaradas como …
#>  3 aeronave                                              Aeronaves declaradas.  
#>  4 apartamento                                           Apartamentos declarado…
#>  5 aplicacao_de_renda_fixa_cdb_rdb_e_outros              Aplicações de renda fi…
#>  6 bem_relacionado_com_o_exercicio_da_atividade_autonoma Bens de atividade autô…
#>  7 benfeitorias                                          Benfeitorias declarada…
#>  8 caderneta_de_poupanca                                 Saldo de cadernetas de…
#>  9 casa                                                  Casas declaradas.      
#> 10 consorcio_nao_contemplado                             Consórcios não contemp…
#> # ℹ 51 more rows
```

### 3. Download and Clean Data

Download the complete historical series for a section. The function
handles the connection, downloads a temporary file, and structures the
data into a tidy format:

``` r
# This will download and clean the "Assets and Rights" dataset
get_irpf("bens_e_direitos")
```

## Data Scale Note

Please note that financial values in the original RFB files are often
scaled (where numbers before the comma represent millions). `irpfR`
automatically detects financial columns and multiplies them by one
million to return absolute values in Reais (BRL), while keeping integer
columns.

## License

This package is licensed under the MIT License.

map_url_irpf <- function(section) {
  url <- irpf_urls_dictionary[[section]]

  if (is.null(url) || url == "") {
    mapped_sections <- names(irpf_urls_dictionary)[sapply(irpf_urls_dictionary, function(x) x != "")]
    stop("Section not found or URL not mapped yet. Available: ",
         paste(mapped_sections, collapse = ", "), call. = FALSE)
  }

  return(url)
}

#' List Available IRPF Data Sections
#'
#' This function returns a table containing all the available data sections
#' that can be retrieved from the Great Numbers of Brazilian Individual Income Tax (IRPF) through this package.
#'
#' @return A tibble with two columns: \code{secao} (the key to be used in other functions)
#' and \code{descricao} (a brief explanation of the data).
#' @export
#'
#' @examples
#' get_sections()

get_sections <- function() {
  message("Fetching available sections for IRPF data...")
  return(irpf_sections_catalog)
}

test_that("get_sections returns a valid tibble", {
  res <- get_sections()

  expect_s3_class(res, "data.frame")

  expect_named(res, c("secao", "descricao"))

  expect_true("bens_e_direitos" %in% res$secao)
})

#' Download and Clean IRPF Data
#'
#' This function connects to the Brazilian Federal Revenue (RFB) open data server,
#' downloads the CSV for the requested section, and performs data cleaning
#' (standardizing names, pivoting to tidy format, and adjusting currency scales).
#'
#' @param section A string representing the section name (e.g., "bens_e_direitos").
#'
#' @return A tidy tibble with columns: \code{ano_calendario}, \code{atributo}, and \code{valor}.
#' @export
#'
#' @examples
#' \donttest{
#'  df <- get_irpf("bens_e_direitos")
#'  head(df)
#' }

get_irpf <- function(section) {
  target_url <- map_url_irpf(section)

  message("Accessing Brazilian Federal Revenue database for section: '", section, "'...")

  temp_file <- tempfile(fileext = ".csv")
  req <- httr2::request(target_url)

  message("Downloading file (this may take a while depending on your connection)...")
  tryCatch({
    httr2::req_perform(req, path = temp_file)
  }, error = function(e) {
    stop("Failed to download data from RFB.\nOriginal error: ", e$message, call. = FALSE)
  })

  message("Download complete! Cleaning and structuring data...")

  raw_data <- suppressMessages({
    readr::read_csv2(
      temp_file,
      locale = readr::locale(encoding = "UTF-8", decimal_mark = ",", grouping_mark = "."),
      na = c("", "NA", "-", "*", "X"),
      show_col_types = FALSE
    )
  })

  clean_data <- raw_data %>%
    janitor::clean_names() %>%

    tidyr::pivot_longer(
      cols = -c(dplyr::any_of("ano_calendario"), where(is.character)),
      names_to = "atributo",
      values_to = "valor",
      values_drop_na = TRUE
    ) %>%

    dplyr::mutate(
      valor = dplyr::if_else(
        grepl("^quantidade", .data$atributo),
        .data$valor,
        .data$valor * 1e6
      )
    )

  message("Data ready for analysis!")

  return(clean_data)
}

#' Get Metadata for a Specific IRPF Section
#'
#' Returns a detailed dictionary of all attributes (columns) present in a
#' specific IRPF data section.
#'
#' @param section A string representing the section name (see \code{get_sections()}).
#'
#' @return A tibble with columns \code{atributo} and \code{descricao}.
#' @export
#'
#' @examples
#' get_metadata("bens_e_direitos")

get_metadata <- function(section) {
  meta <- irpf_metadata_storage[[section]]

  if (is.null(meta)) {
    stop("Metadata not found for section: ", section,
         ". Run `get_sections()` to check valid keys.", call. = FALSE)
  }

  message("Fetching metadata for section: '", section, "'...")
  return(meta)
}

---
title: "Introduction to irpfR"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to irpfR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The ```irpfR``` package provides a high-level interface to access and clean the Personal Income Tax (IRPF) Open Data from the Brazilian Federal Revenue (Receita Federal do Brasil).
This vignette demonstrates the typical workflow: discovering available datasets, understanding their attributes through built-in metadata, and downloading cleaned data for analysis.

## 1. Discovering Available Data

The Brazilian Federal Revenue publishes data in several "sections" (e.g., assets, debts, income brackets). You can list all sections currently supported by the package using ```get_sections()```:

```{r}
library(irpfR)

# List available datasets
sections <- get_sections()
head(sections)
```

## 2. Inspecting Metadata

Government CSV files often have cryptic column names or complex tax definitions. To understand the content of a section before downloading it, use get_metadata():

```{r}
# Get descriptions for the "Assets and Rights" (Bens e Direitos) section
metadata <- get_metadata("bens_e_direitos")
head(metadata)
```

## 3. Downloading and Cleaning Data

The core function of the package is get_irpf(). It performs several automated engineering tasks:

* **Download**: Connects to the RFB servers and handles the file transfer.
* **Encoding**: Corrects UTF-8/Latin1 issues common in Brazilian government files.
* **Tidying**: Converts "wide" tables into a "long" (tidy) format, making them ready for ggplot2 or dplyr.
* **Smart Scaling**: Financial values are converted from millions to absolute BRL, while counts (like number of taxpayers) remain as raw integers.

```{r}
# Download data for "Assets and Rights"
df_bens <- get_irpf("bens_e_direitos")

# The resulting data is tidy
# Columns: ano_calendario, atributo, valor
head(df_bens)
```

## Why use irpfR?
Directly reading raw files from the government portal can be challenging due to inconsistent decimal marks (using commas), non-standard NA characters (like ```-``` or ```*```), and varying numerical scales.```irpfR``` encapsulates all these rules, allowing researchers to focus on the economic analysis rather than data cleaning.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{get_metadata}
\alias{get_metadata}
\title{Get Metadata for a Specific IRPF Section}
\usage{
get_metadata(section)
}
\arguments{
\item{section}{A string representing the section name (see \code{get_sections()}).}
}
\value{
A tibble with columns \code{atributo} and \code{descricao}.
}
\description{
Returns a detailed dictionary of all attributes (columns) present in a
specific IRPF data section.
}
\examples{
get_metadata("bens_e_direitos")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sections.R
\name{get_sections}
\alias{get_sections}
\title{List Available IRPF Data Sections}
\usage{
get_sections()
}
\value{
A tibble with two columns: \code{secao} (the key to be used in other functions)
and \code{descricao} (a brief explanation of the data).
}
\description{
This function returns a table containing all the available data sections
that can be retrieved from the Great Numbers of Brazilian Individual Income Tax (IRPF) through this package.
}
\examples{
get_sections()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extracao.R
\name{get_irpf}
\alias{get_irpf}
\title{Download and Clean IRPF Data}
\usage{
get_irpf(section)
}
\arguments{
\item{section}{A string representing the section name (e.g., "bens_e_direitos").}
}
\value{
A tidy tibble with columns: \code{ano_calendario}, \code{atributo}, and \code{valor}.
}
\description{
This function connects to the Brazilian Federal Revenue (RFB) open data server,
downloads the CSV for the requested section, and performs data cleaning
(standardizing names, pivoting to tidy format, and adjusting currency scales).
}
\examples{
\donttest{
 df <- get_irpf("bens_e_direitos")
 head(df)
}
}
